Package: RUnit
Version: 0.4.28
Date: 2014-09-19
Title: R Unit Test Framework
Author: Matthias Burger <burgerm@users.sourceforge.net>, Klaus
        Juenemann <k.junemann@gmx.net>, Thomas Koenig
        <thomas.koenig@epigenomics.com>
Maintainer: ORPHANED
LazyLoad: yes
Depends: R (>= 2.5.0), utils (>= 2.5.0), methods (>= 2.5.0)
Description: R functions implementing a standard Unit Testing
        framework, with additional code inspection and report
        generation tools
License: GPL-2
Packaged: 2015-01-06 11:32:32 UTC; ripley
NeedsCompilation: no
X-CRAN-Original-Maintainer: Matthias Burger
        <burgerm@users.sourceforge.net>
X-CRAN-Comment: Orphaned and updated by the CRAN team on 2014-09-19 as
        vignette locations were never updated for R 3.1.0.
Repository: CRAN
Date/Publication: 2015-01-06 19:48:43
Built: R 4.4.0; ; 2026-02-17 20:09:05 UTC; unix
