Package: RUnit
Version: 0.4.33.1
Date: 2024-02-09
Title: R Unit Test Framework
Authors@R: c(person(given = "Matthias",
                    family = "Burger",
                    role = "aut",
                    email = "burgerm@users.sourceforge.net"),
             person(given = "Klaus",
                    family = "Juenemann",
                    role = "aut",
                    email = "k.junemann@gmx.net"),
             person(given = "Thomas",
                    family = "Koenig",
                    role = "aut",
                    email = "thomas.koenig@epigenomics.com"),
             person(given = "Roman",
                    family = "Zenka",
                    role = "cre",
                    email = "zenka.roman@mayo.edu"))
LazyLoad: yes
Depends: R (>= 2.5.0), utils (>= 2.5.0), methods (>= 2.5.0), graphics
        (>= 2.5.0)
Suggests: XML (>= 3.1.0)
Description: R functions implementing a standard Unit Testing
        framework, with additional code inspection and report
        generation tools.
License: GPL-2
NeedsCompilation: no
Packaged: 2025-06-17 10:32:59 UTC; hornik
Repository: CRAN
Date/Publication: 2025-06-17 10:53:27 UTC
Author: Matthias Burger [aut],
  Klaus Juenemann [aut],
  Thomas Koenig [aut],
  Roman Zenka [cre]
Maintainer: Roman Zenka <zenka.roman@mayo.edu>
