######################################################################
##
## RUnit
## =====================================
##
## inst/NAMESPACE
## =====================================
## initialization of classes, namespace, ...
##
##
##
##
## Version
## =====================================
##  $Id: NAMESPACE,v 1.7 2009/04/22 13:46:38 burgerm Exp $
##
##
######################################################################

import(utils)
import(methods)

export(".setUp",
       ".tearDown",
       "checkTrue",
       "checkEquals",
       "checkEqualsNumeric",
       "checkException",
       "checkIdentical",
       "DEACTIVATED",
       "defineTestSuite",
       "getErrors",
       "inspect",
       "isValidTestSuite",
 #      "print.RUnitTestData",
       "printTextProtocol",
       "printHTMLProtocol",
       "printHTML.trackInfo",
       "runTestSuite",
       "runTestFile",
 #      "summary.RUnitTestData",
       "tracker")


S3method(print, RUnitTestData)
S3method(summary, RUnitTestData)
