This virtual service provides a facility to make runscripts that depends on a
syslog daemon portable across different systems, for instance when
the logger daemon name is not known in advance.
This service is not a logger daemon itself, and does not provide runscripts
for syslog daemons.

Usage:

 * make sure this service is enabled in your rundirectory

  # update-service --add /etc/sv/default-syslog

 * to make sure that a syslog daemon is avaliable before your service is up
    add the following line in the run file of your service

      'sv start default-syslog  &&  sv check default-syslog  ||  exit 170'

Given that the trick is done in the check file, the sleep time in the default-syslog
service can be raised at will. The list of supported syslog daemons is available
in the check file.

Limits:

 * the default-syslog service only work if:
    1. at least one native runit service exists for a syslog daemon
    2. the name of the native runit service match the one in the check file
    3. the syslog daemon is enabled in the current rundirectory

 as long as the syslog service that you are using is only available as sysvinit
 script, this service is useless and should be disabled.
 Note that when runit is init, sysvscripts are always started before runsvdir, so
 as long as no native runscript is available for the syslog daemon, it is possible
 to skip the check for syslog in any runscript.

 
